;;;
;;; irchat-pj-coding-system.el
;;; last modify : Tue, 12 Jan 1999 by simm-irc@fan.gr.jp / irchat-PJ Project
;;; (date format modification by simm-irc@fan.gr.jp, Sun, 27 Jun 1999)
;;; see file irchat-copyright.el for change log and copyright info
;;;

(provide 'irchat-pj-coding-system)
(require 'irchat-pj-jisx0201)

;; begin: from irchat-2.4jp24g
(defun irchat-define-service-coding-system (service)
  (if (fboundp 'modify-coding-system-alist)
      (modify-coding-system-alist 'network service
				  (cons 'no-conversion 'no-conversion))
    (if (fboundp 'define-service-coding-system)
	(define-service-coding-system service nil (cons *noconv* *noconv*))
      (if (fboundp 'define-service-kanji-code)
	  (define-service-kanji-code service nil 0)))))

(defun irchat-set-process-coding-system (proc)
  (if (fboundp 'modify-coding-system-alist)
      (set-process-coding-system proc 'no-conversion 'no-conversion)
    (if (fboundp 'set-process-coding-system)
	(set-process-coding-system proc *noconv* *noconv*)
      (if (fboundp 'set-process-kanji-code)
	  (set-process-kanji-code proc 0)))))
;; end: from irchat-2.4jp24g

(defun irchat-pj-string-out-iso-2022-7bit-ss2 (str)
  (if (fboundp 'encode-coding-string)
      (if (boundp '*iso-2022-ss2-7*)
	  (encode-coding-string str *iso-2022-ss2-7*)
	(encode-coding-string str 'iso-2022-7bit-ss2))
    (if (fboundp 'code-convert-string)
	(code-convert-string str *internal* *junet*)
      (if (fboundp 'convert-string-kanji-code)
	  (let ((orig-to-ascii-process to-ascii-process)
		(orig-to-kanji-process to-kanji-process))
	    (setq to-ascii-process ?B)
	    (setq to-kanji-process ?B)
	    (setq str (convert-string-kanji-code str 3 2))
	    (setq to-ascii-process orig-to-ascii-process)
	    (setq to-kanji-process orig-to-kanji-process)
	    str)
	str))))

(defun irchat-pj-string-out-euc-jp (str)
  (if (fboundp 'encode-coding-string)
      (if (boundp '*euc-jp*)
	  (encode-coding-string str *euc-jp*)
	(encode-coding-string str 'euc-japan))
    (if (fboundp 'code-convert-string)
	(code-convert-string str *internal* *euc-jp*)
      str)))

(defun irchat-pj-string-out-sjis (str)
  (if (fboundp 'encode-coding-string)
      (if (boundp '*sjis*)
	  (encode-coding-string str *sjis*)
	(encode-coding-string str 'sjis))
    (if (fboundp 'code-convert-string)
	(code-convert-string str *internal* *sjis*)
      (if (fboundp 'convert-string-kanji-code)
	  (let ((orig-to-ascii-process to-ascii-process)
		(orig-to-kanji-process to-kanji-process))
	    (setq to-ascii-process ?B)
	    (setq to-kanji-process ?B)
	    (setq str (convert-string-kanji-code str 3 1))
	    (setq to-ascii-process orig-to-ascii-process)
	    (setq to-kanji-process orig-to-kanji-process)
	    str)
	str))))

(defun irchat-pj-string-in-iso-2022-7bit-ss2 (str)
  (if (fboundp 'decode-coding-string)
      (if (boundp '*iso-2022-ss2-7*)
          (decode-coding-string str *iso-2022-ss2-7*)
        (decode-coding-string str 'iso-2022-7bit-ss2))
    (if (fboundp 'code-convert-string)
        (code-convert-string str *junet* *internal*)
      (if (fboundp 'convert-string-kanji-code)
          (convert-string-kanji-code str 2 3)
        str))))

(defun irchat-pj-string-in-euc-jp (str)
  (let ((len (length str)) (ch 0) (ps 0) (pt 0) (msb nil) (slist nil))
    ;; Phase 1: split EUC code and others
    (while (< pt len)
      (setq ch (elt str pt))
      ;;(message "%d" ch)
      (cond ((and (eq nil msb) (>= ch 128))
             (setq slist (append slist (list (substring str ps pt))))
             (setq msb t
                   ps pt
                   pt (+ 2 pt)))
            ((and (eq t msb) (< ch 128))
             (setq slist (append slist (list (substring str ps pt))))
             (setq msb nil
                   ps pt
                   pt (1+ pt)))
            ((eq nil msb)
             (setq pt (1+ pt)))
            ((eq t msb)
             (setq pt (+ 2 pt)))))
    (setq slist (append slist (list (substring str ps len))))
    ;; Phase 2: convert string
    (setq str "")
    (while slist
      (setq str (concat str (if (< (elt (car slist) 0) 128)
                                (irchat-pj-string-in-iso-2022-7bit-ss2 (car slist))
                              (irchat-pj-string-in-euc-jp-internal (car slist))))
            slist (cdr slist)))
    str))

(defun irchat-pj-string-in-euc-jp-internal (str)
  (if (fboundp 'decode-coding-string)
      (if (boundp '*euc-jp*)
	  (decode-coding-string str *euc-jp*)
	(decode-coding-string str 'euc-japan))
    (if (fboundp 'code-convert-string)
	(code-convert-string str *euc-jp* *internal*)
      str)))

(defun irchat-pj-string-in-sjis (str)
  (let ((len (length str)) (ch 0) (ps 0) (pt 0) (msb nil) (slist nil))
    ;; Phase 1: split SJIS code and others
    (while (< pt len)
      (setq ch (elt str pt))
      (cond ((and (eq nil msb) (>= ch 128))
             (setq slist (append slist (list (substring str ps pt))))
             (setq msb t
                   ps pt
                   pt (1+ pt))
             (if (or (and (<= 128 ch) (<= ch 159)) (<= 224 ch)) ;; 2bytes
                 (setq pt (1+ pt))))
            ((and (eq t msb) (< ch 128))
             (setq slist (append slist (list (substring str ps pt))))
             (setq msb nil
                   ps pt
                   pt (1+ pt)))
            (t
             (setq pt (1+ pt))
             (if (or (and (<= 128 ch) (<= ch 159)) (<= 224 ch)) ;; 2bytes
                 (setq pt (1+ pt))))))
    (setq slist (append slist (list (substring str ps len))))
    ;; Phase 2: convert string
    (setq str "")
    (while slist
      (setq str (concat str (if (< (elt (car slist) 0) 128)
                                (irchat-pj-string-in-iso-2022-7bit-ss2 (car slist))
                              (irchat-pj-string-in-sjis-internal (car slist))))
            slist (cdr slist)))
    str))

(defun irchat-pj-string-in-sjis-internal (str)
  (if (fboundp 'decode-coding-string)
      (if (boundp '*sjis*)
	  (decode-coding-string str *sjis*)
	(decode-coding-string str 'sjis))
    (if (fboundp 'code-convert-string)
	(code-convert-string str *sjis* *internal*)
      (if (fboundp 'convert-string-kanji-code)
	  (convert-string-kanji-code str 1 3)
	str))))

;; begin: from irchat-pj-filter-convert-code in irchat-pj 2.4.23.19
(defun irchat-pj-string-in-auto-detect-primitive (str)
  (let ((len (length str))(pp 0)(ps 0)(pe 0)(fl 0)(code nil))
    ;; Phase 1: check MSB
    (while (and (< pp len) (eq nil code))
      (setq ps (elt str pp)
            pp (1+ pp))
      (if (<= 128 ps)
          (setq code t)))
    (if (eq nil code)
        'iso-2022-7bit-ss2
      (setq pp 0
            code nil)
      (while (null code)
        (setq ps (elt str pp)
              pp (1+ pp))
        (if (= pp len)
            (setq code 'unknown))
        (cond ((or (= 13 ps) (= 10 ps))      ;; CRLF
               (setq code 'unknown))
              ((and (<= 128 ps) (<= ps 159)) ;; 0x80 <= ch <= 0x9f
               (setq code 'sjis))
              ((and (= pe 1) (<= ps 127))    ;; second byte MSB = 0
               (setq code 'sjis))
              ((and (= 2 pe) (<= 161 ps) (<= ps 243)) ;; second byte of EUC kana
               (setq fl 2)
               (setq pe 0))
              ((and (= 0 pe) (= 164 ps))     ;; first byte of EUC kana
               (setq pe 2))
              ((<= 160 ps)                   ;; 0xa0 <= ch
               (if (null (= 2 fl ))
                   (setq fl 1))
               (if (= 2 pe)
                   (setq code 'sjis)
                 (setq pe (- 1 pe))))))
      (if (eq 'unknown code)
          (if (= 2 fl) 'euc-jp 'sjis)
        code))))

(defun irchat-pj-string-in-auto-detect (str)
  "Decide code of argument string."
  (let ((code (irchat-pj-string-in-auto-detect-primitive str)))
    (cond ((eq 'euc-jp code)
	   (if irchat-pj-code-verbose
	       (message "EUC-JP code detected."))
	   (irchat-pj-string-in-euc-jp str))
	  ((eq 'sjis code)
	   (if irchat-pj-code-verbose
	       (message "SJIS code detected."))
	   (irchat-pj-string-in-sjis str))
	  ((eq 'iso-2022-7bit-ss2 code)
	   (irchat-pj-string-in-iso-2022-7bit-ss2 str))
	  (t
	   str))))

(defun irchat-pj-string-in-non-euc (str)
  "Decide code of argument string."
  (let ((code (irchat-pj-string-in-auto-detect-primitive str)))
    (cond ((or (eq 'sjis code)(eq 'euc-jp code))
	   (if irchat-pj-code-verbose
	       (message "SJIS code detected."))
	   (irchat-pj-string-in-sjis str))
	  ((eq 'iso-2022-7bit-ss2 code)
	   (irchat-pj-string-in-iso-2022-7bit-ss2 str))
	  (t
	   str))))
;; end: from irchat-pj-filter-convert-code in irchat-pj 2.4.23.19

;; begin: modified by simm-irc@fan.gr.jp
(defun irchat-pj-string-in-convert (str)
  (cond ((eq irchat-pj-string-in-code 'noconv) str)
	((or (eq irchat-pj-string-in-code 'euc-jp)
	     (eq irchat-pj-string-in-code 'euc-japan))
	 (irchat-pj-string-in-euc-jp str))
	((or (eq irchat-pj-string-in-code 'sjis)
	     (eq irchat-pj-string-in-code 'shift-jis)
	     (eq irchat-pj-string-in-code 'ms-kanji))
	 (irchat-pj-string-in-sjis str))
	((or (eq irchat-pj-string-in-code 'iso-2022-7bit-ss2)
	     (eq irchat-pj-string-in-code 'jis))
	 (irchat-pj-string-in-iso-2022-7bit-ss2 str))
	(t
	 (irchat-pj-string-in-auto-detect str))))

(defun irchat-pj-string-out-convert (str)
  (cond ((eq irchat-pj-string-out-code 'noconv) str)
	((or (eq irchat-pj-string-out-code 'euc-jp)
	     (eq irchat-pj-string-out-code 'euc-japan))
	 (irchat-pj-string-out-euc-jp str))
	((or (eq irchat-pj-string-out-code 'sjis)
	     (eq irchat-pj-string-out-code 'shift-jis)
	     (eq irchat-pj-string-out-code 'ms-kanji))
	 (irchat-pj-string-out-sjis str))
	((or (eq irchat-pj-string-out-code 'iso-2022-7bit-ss2)
	     (eq irchat-pj-string-out-code 'jis))
	 (irchat-pj-string-out-iso-2022-7bit-ss2 str))
	(t
	 (irchat-pj-string-out-iso-2022-7bit-ss2 str))))

(defun irchat-string-in (str)
  (if str
      (or (irchat-pj-string-in-convert str) str) nil))

(defun irchat-string-out (str)
  (if str
      (or (irchat-pj-string-out-convert str) str) nil))
;; end: modified by simm-irc@fan.gr.jp
